import * as React from "react";
import { Bounds } from '@use-gesture/react';
/**
 * Touchable is a low level component which implements
 * the useTouchable hook. It provides consistent hover and
 * active styles on desktop and mobile devices.
 */
export interface DragbleProps {
    disabled?: boolean;
    /** By default, a touchable element will only highlight after a short
     * delay to prevent unintended highlights when scrolling. This will
     * only effect touch devices. */
    delay?: number;
    /**
     * By default, a touchable element will have an expanded press area of 20px
     * on touch devices. This will only effect touch devices.
     */
    pressExpandPx?: number;
    /** Whether active presses should be terminated when scrolling occurs. Typically this should be true. */
    terminateOnScroll?: boolean;
    component?: React.ElementType<any>;
    bounds?: Bounds | HTMLElement | {
        current: HTMLElement | null;
    };
    [key: string]: any;
}
/** 随意拽按的一个区域： Dragble ,限制走动范围区域ref[div.ref.getBounRects]
 * 点击触发还是放入内部嵌套的组件中做。
 */
export declare const Dragable: React.ForwardRefExoticComponent<Pick<DragbleProps, keyof DragbleProps> & React.RefAttributes<HTMLDivElement>>;
interface DragPinnerProps extends React.HTMLAttributes<HTMLDivElement> {
    /**延迟关闭执行 onRequestPress；    =0不会自己关闭执行。
     * delay这段时间内，也没办法点击页面按钮的,也要同步等待后端的应答返回来，用户才能继续操作。
     *  */
    delay?: number;
    size?: "xs" | "sm" | "md" | "lg" | "xl";
    /** Attempt to center the spinner in the parent element */
    center?: boolean;
    /** Use an optional label */
    label?: string;
    doing: boolean;
    /**点击关闭 或者任意其它的回调处理  */
    onRequestPress: () => void;
    bounds?: Bounds | HTMLElement | {
        current: HTMLElement | null;
    };
}
/** 一个例子
 *防止背景滚动 https://www.it610.com/article/1294815963329404928.htm 需要对ref.Div的ontouchstart、ontouchmove两个都加上preventDefault();
 * */
export declare const DragablePinner: React.FunctionComponent<DragPinnerProps>;
export {};
