import { jsx as _jsx } from "@emotion/react/jsx-runtime";
import * as React from "react";
// import { useTheme } from "./Theme/Providers";
// import { mergeRefs } from "./Hooks/merge-refs";
import { useFloating, autoUpdate, offset, flip, shift, useClick, useDismiss, useRole, useInteractions, FloatingPortal, FloatingFocusManager, useId } from "@floating-ui/react";
import { mergeRefs } from "react-merge-refs";
import { useTheme } from "./Theme/Providers";
/*
非对话框的：tooltip menu dropdownList popover select option 这几个才是 floating-ui 真正的用途！ 其实对于Dialog对话框没多大用。
这里useFloating生成的{...data}传递到底下的互动元素css样式<div  className="PopoverContent"  position: state. , top:
*/
export function usePopover({ initialOpen = false, placement, modal = false, open: controlledOpen, fixed = false, onOpenChange: setControlledOpen } = {}) {
    const [uncontrolledOpen, setUncontrolledOpen] = React.useState(initialOpen);
    const [labelId, setLabelId] = React.useState();
    const [descriptionId, setDescriptionId] = React.useState();
    const open = controlledOpen ?? uncontrolledOpen;
    const setOpen = setControlledOpen ?? setUncontrolledOpen;
    const data = useFloating({
        placement,
        strategy: fixed ? 'fixed' : 'absolute',
        open,
        onOpenChange: setOpen,
        whileElementsMounted: autoUpdate,
        middleware: [offset(5), flip(), shift()]
    });
    const context = data.context;
    const click = useClick(context, {
        enabled: controlledOpen == null
    });
    const dismiss = useDismiss(context, {
        outsidePress: !modal,
    });
    const role = useRole(context);
    const interactions = useInteractions([click, dismiss, role]);
    return React.useMemo(() => ({
        open,
        setOpen,
        ...interactions,
        ...data,
        modal,
        labelId,
        descriptionId,
        setLabelId,
        setDescriptionId
    }), [open, setOpen, interactions, data, modal, labelId, descriptionId]);
}
export const PopoverContext = React.createContext(null);
export const usePopoverState = () => {
    const context = React.useContext(PopoverContext);
    if (context == null) {
        throw new Error("Popover components must be wrapped in <Popover />");
    }
    return context;
};
/*为适应横屏手机：限制内容高度，不应该超过4行的输入框高度。
类似Dialog 但一般页面占用尺寸较小的。默认点击外部关闭。默认跟随ref定位的。
例子  https://codesandbox.io/s/distracted-swirles-jo1pvu?file=/src/App.tsx
旧的Sheet不能自适应也不配套TwoHalfFrame已经改用Popover替代;
遇到高度超出的，在html<body/>要加 overflow-y: auto; 若内容height太大自适应滚动就失去感觉了？还是BUG。
【特别注意】内容太大的，高度超出横屏手机屏幕height的情况，一定请改用Dialog; 高度超了Popover可能会出问题。
若没有加<PopoverRefer></PopoverRefer>导致缺省定位屏幕左上角,此时placement无效。高度大的不加<PopoverRefer滚动好控制。
* */
export function Popover({ children, modal = false, ...restOptions }) {
    // This can accept any props as options, e.g. `placement`,
    // or other positioning options.
    const popover = usePopover({ modal, ...restOptions });
    return (_jsx(PopoverContext.Provider, { value: popover, children: children }));
}
//popover对话框跟随参考位置的DIV；  以及可选的 绑定onClick开启显示。
//没有配套PopoverRefer的情况：默认显示在左上角位置。
export const PopoverRefer = React.forwardRef(function PopoverRefer({ children, asChild = false, ...props }, propRef) {
    const state = usePopoverState();
    const childrenRef = children.ref;
    //这个state.reference其实是在useFloating()返回==ref;
    const ref = React.useMemo(() => mergeRefs([state.reference, propRef, childrenRef]), [state.reference, propRef, childrenRef]);
    //自定义版： `asChild` allows the user to pass any element as the anchor
    if (asChild && React.isValidElement(children)) {
        return React.cloneElement(children, state.getReferenceProps({
            ref,
            ...props,
            ...children.props,
            "data-state": state.open ? "open" : "closed"
        }));
    }
    //正常的按钮版；
    return (_jsx("button", { ref: ref, "data-state": state.open ? "open" : "closed", ...state.getReferenceProps(props), children: children }));
});
//主体部分:
export const PopoverContent = React.forwardRef(function PopoverContent(props, propRef) {
    const state = usePopoverState();
    const ref = React.useMemo(() => mergeRefs([state.floating, propRef]), [
        state.floating,
        propRef
    ]);
    const theme = useTheme();
    //modal state.y 实际来自于usePopover();
    return (_jsx(FloatingPortal, { children: state.open && (_jsx(FloatingFocusManager, { context: state.context, modal: state.modal, children: _jsx("div", { className: "PopoverContent", ref: ref, style: {
                    zIndex: theme.zIndices.popover,
                    position: state.strategy,
                    top: state.y ?? 0,
                    left: state.x ?? 0,
                    //width: "max-content",没法自适应宽度没法Wrap;
                    backgroundColor: 'white',
                    boxShadow: theme.shadows.sm,
                    border: '1px solid #ddd',
                    padding: '4px 8px',
                    borderRadius: '4px',
                    ...props.style
                }, "aria-labelledby": state.labelId, "aria-describedby": state.descriptionId, ...state.getFloatingProps(props), children: props.children }) })) }));
});
//仅仅id标记不同于内容区的；
export const PopoverHeading = React.forwardRef(function PopoverHeading({ children, ...props }, ref) {
    const { setLabelId } = usePopoverState();
    const id = useId();
    // Only sets `aria-labelledby` on the Popover root element
    // if this component is mounted inside it.
    React.useLayoutEffect(() => {
        setLabelId(id);
        return () => setLabelId(undefined);
    }, [id, setLabelId]);
    return (_jsx("h4", { ...props, ref: ref, id: id, children: children }));
});
//增加一层就为了添加一个w3c设置： aria-describedby={state.descriptionId} 屏幕阅读器 ：听觉UI@ 用途的。
export const PopoverDescription = React.forwardRef(function PopoverDescription({ children, ...props }, ref) {
    const { setDescriptionId } = usePopoverState();
    const id = useId();
    // Only sets `aria-describedby` on the Popover root element
    // if this component is mounted inside it.
    React.useLayoutEffect(() => {
        setDescriptionId(id);
        return () => setDescriptionId(undefined);
    }, [id, setDescriptionId]);
    const theme = useTheme();
    return (_jsx("div", { className: "PopoverDescription", ...props, ref: ref, id: id, style: {
            display: "flex",
            flexWrap: 'wrap',
            //zIndex: theme.zIndices.popover,
        }, children: children }));
});
export const PopoverClose = React.forwardRef(function PopoverClose({ children, ...props }, ref) {
    const state = usePopoverState();
    return (_jsx("button", { style: {
            display: "block",
            margin: 'auto',
        }, ...props, ref: ref, onClick: () => state.setOpen(false), children: children }));
});
//# sourceMappingURL=Popover.js.map